function outImage = ahe2(inImage, numofBin, windowSize, minv, maxv)
% Adaptive histogram equalization.

%inImage = double(inImage);
if mod(windowSize, 2) == 0
    windowSize = windowSize + 1;
end;

blockImage = zeros(windowSize);
[row, col] = size(inImage);
outImage = zeros(row, col);
rowofWindow = ceil(row/windowSize);
colofWindow = ceil(col/windowSize);

cumuMap = cell(rowofWindow, colofWindow);


for i = 1:rowofWindow
    for j = 1:colofWindow
        rowArray = mod((i-1)*windowSize+1:i*windowSize, row);
        colArray = mod((j-1)*windowSize+1:j*windowSize, col);
        rowArray(find(rowArray == 0)) = row;
        colArray(find(colArray == 0)) = col;
        blockImage = inImage(rowArray, colArray);
        cumuMap{i,j} = Cumu(blockImage, numofBin, minv, maxv);
    end;
end;
K = (double(maxv) - double(minv)+1)/double(numofBin);  % number of gray levels in one bin.
for i = 1:row
    for j = 1:col
        bin = floor((inImage(i,j) - minv)/K);
        if i <= windowSize | j <=windowSize | i > row - windowSize | j > col - windowSize
            curTileRow = ceil(i/windowSize);
            curTileCol = ceil(j/windowSize);
            
            if (inImage(i,j) >= minv & inImage(i,j) <= maxv)
                 
                 cumu = cumuMap{curTileRow, curTileCol};
                 if bin == numofBin
                    outImage(i,j) =cumu(bin) * (maxv - minv) + minv;
                else
                    outImage(i,j) = cumu(bin+1) * (maxv - minv) + minv;
                end;           
            else
                outImage(i,j) = inImage(i,j);
            end;
        else
            %find 4 neighbor tile's center
            % also find the center of the neighbor tile.
            tile_x = floor(i/windowSize);
            tile_y = floor(j/windowSize);
            if mod(i, windowSize) <= (windowSize+1)/2
                tile_xup = tile_x; tile_xdown = tile_x + 1;
            else
                tile_xup = tile_x + 1; tile_xdown = tile_x + 2;
            end;
            
            if mod(j, windowSize) <= (windowSize + 1)/2
                tile_yleft = tile_y; tile_yright = tile_y + 1;
            else
                tile_yleft = tile_y + 1; tile_yright = tile_y + 2;
            end;
            
            tileUL = [tile_xup tile_yleft];
            centerUL = [(tileUL(1)-1) * windowSize + (windowSize + 1)/2,...
                (tileUL(2)-1) * windowSize + (windowSize + 1)/2];

            tileUR = [tile_xup tile_yright];
            centerUR = [(tileUR(1)-1) * windowSize + (windowSize + 1)/2,...
                (tileUR(2)-1) * windowSize + (windowSize + 1)/2];

            tileLL = [tile_xdown tile_yleft];
            centerLL = [(tileLL(1)-1) * windowSize + (windowSize + 1)/2,...
                (tileLL(2)-1) * windowSize + (windowSize + 1)/2];

            tileLR = [tile_xdown tile_yright];
            centerLR = [(tileLR(1)-1) * windowSize + (windowSize + 1)/2,...
                (tileLR(2)-1) * windowSize + (windowSize + 1)/2];

            % x: i
            % y: j
            % x1: centerUL(1)
            % x2: centerLL(1)
            % y1: centerUL(2)
            % y2: centerUR(2)

            % Get the value in the bilinear equation, just for convenience.
            x2_x1 = centerLL(1) - centerUL(1);
            x2_x = centerLL(1) - i;
            y2_y1 = centerUR(2) - centerUL(2);
            y2_y = centerUR(2) - j;
            x_x1 = i - centerUL(1);
            y_y1 = j - centerUL(2);
            cumuQ11 = cumuMap{tile_xup, tile_yleft};
            cumuQ12 = cumuMap{tile_xup, tile_yright};
            cumuQ21 = cumuMap{tile_xdown, tile_yleft};
            cumuQ22 = cumuMap{tile_xdown, tile_yright};
            if bin == numofBin
                    fQ11 =cumuQ11(bin) * (maxv - minv) + minv;
                    fQ12 =cumuQ12(bin) * (maxv - minv) + minv;
                    fQ21 =cumuQ21(bin) * (maxv - minv) + minv;
                    fQ22 =cumuQ22(bin) * (maxv - minv) + minv;
            else
                    fQ11 =cumuQ11(bin+1) * (maxv - minv) + minv;
                    fQ12 =cumuQ12(bin+1) * (maxv - minv) + minv;
                    fQ21 =cumuQ21(bin+1) * (maxv - minv) + minv;
                    fQ22 =cumuQ22(bin+1) * (maxv - minv) + minv;
            end;

            outImage(i,j) = fQ11 * x2_x * y2_y / (x2_x1 * y2_y1) + ...
                fQ21 * x_x1 * y2_y /( x2_x1 * y2_y1) + ...
                fQ12 * (x2_x) * (y_y1) /(x2_x1 * y2_y1) + ...
                fQ22 * (x_x1) * (y_y1) / (x2_x1 * y2_y1);
                        
            end;

    end;
    if mod(i,50) == 0
fprintf('Adaptive Histogram Equalization: row %d\n', i);
    end;
end;

outImage;
        
        
        
        
        
        
     