function freq = histogram_clahe(myimage, n, min, max, limit)

% Compute histogram of an image. This is CLAHE version.
% Input: I is the file name which contains a integers or float point image
% format (have to be grayscale). n is the number of bin. min and max is 
% the minima and maximal value of the image. 

% Output: freq is vector with the frequency of each pixel. 

myimage = double(myimage);
freq = zeros(1,n);
[row, col] = size(myimage);

freeBinPool = 1:n;
currFreeBin = 1;
K = (double(max) - double(min)+1.0)/double(n);  % number of gray levels in one bin.
M = length(find(myimage >=min & myimage <= max));   % number of all pixels btw min and max.
limitPixel = limit * M; % max number of pixel in one bin.
for i = 1:row
    for j = 1:col
        if (myimage(i,j) >= min & myimage(i,j) <= max)
            bin = floor((myimage(i,j) - min)/K);
            if bin == n
                freq(bin) = freq(bin) + 1;
            elseif bin == 0
                freq(1) = freq(1) + 1;
                
            else
                if freq(bin) >= limitPixel
                    toDel = find(freeBinPool == bin);
                    freeBinPool(toDel) = [];
                    freq(freeBinPool(currFreeBin)) = freq(freeBinPool(currFreeBin)) + 1;
                    currFreeBin = mod(currFreeBin + 1, length(freeBinPool));
                    if currFreeBin == 0
                        currFreeBin = length(freeBinPool);
                    end;
                else
                    
                    freq(bin+1) = freq(bin+1) + 1;
                end;
            end;
        end;
    end;
end;

freq = freq/M;
